<?php
/**
 * User: paulopimenta
 * Date: 14/02/14
 * Time: 10:25
 */

namespace Fiscal\Logic\NotaFiscal\Entrada;

use Contabilidade\Logic\Util;
use Financeiro\Entities\Ajctpagar;
use Financeiro\Entities\Contareceber;
use Fiscal\Entities\Contapagar;
use Fiscal\Entities\Notafiscal;
use Fiscal\Logic\NotaFiscal\Exceptions\AprovacaoException;
use Fiscal\Logic\NotaFiscal\ProceduresNF;
use Geral\Entities\Itemproduto;
use Geral\Entities\Lanccont;
use Geral\Entities\Tipoctpagar;
use Geral\Util\Ambiente;
use Geral\Util\DateUtil;
use Planejamento\Entities\Autoforne;
use Setforms2\Shared\Registry;
use Fiscal\Logic\NotaFiscal\ValidaDataIntegracao;

class AprovacaoNFEntrada
{
    private $notafiscal;

    public function __construct(Notafiscal $notafiscal)
    {
        $this->notafiscal = $notafiscal;
    }

    public function validacaoGeral($cdtpctpagop, $idcheck, $cdtpctrecop) {

        $cdmoddocfisc = $this->notafiscal->getCdmoddocfisc();
        $dsnfestcanc = $this->notafiscal->getDsnfestcanc();

        try{
            //
            self::validaModDocFisc($cdmoddocfisc);
            //
            self::validaNotaFiscEstornada($dsnfestcanc);
            //
            self::validaIdErroNotaFisc();
            //
            self::validaTipoDeConta($cdtpctpagop, $idcheck, $cdtpctrecop);
            //
            ValidaDataIntegracao::validaDataIntegracao($this->notafiscal->getCdempresa(), $this->notafiscal->getIdentrsaidop(),
                                                       $this->notafiscal->getCdtipooper(), $this->notafiscal->getDtentrsaid());
            //
//            self::validaConsisteFiscal();

            return true;
        }
        catch (\Exception $e) {
            return $e;
        }
    }

    //
    public function validaModDocFisc($cdmoddocfisc) {
        if ($cdmoddocfisc == null) {
            AprovacaoException::notaFiscSemModDocFic();
        }
    }

    //
    public function validaNotaFiscEstornada($dsnfestcanc) {
        if($dsnfestcanc == 'S') {
            AprovacaoException::notaFiscalEstornada();
        }
    }

    //
    public function validaIdErroNotaFisc() {
        $iderros = $this->notafiscal->getIderros();
        if ($iderros[0] == '1') {
            AprovacaoException::notaFiscalComErrosCadast();
        }
    }

    //
    public function validaTipoDeConta($cdtpctpagop, $idcheck, $cdtpctrecop){
        if ($this->notafiscal->getIdentrsaidop() == Notafiscal::NOTA_FISCAL_ENTRADA) {
            $validaConta[0] = false;
            $tipoDeConta    = Itemproduto::getRepository()->getTipoContaPagarProd($this->notafiscal->getCdfilial(), $this->notafiscal->getNrlanctonf(), $this->notafiscal->getNrorg());
            $produtosNota   = Itemproduto::findByCdfilialAndNrlanctonfAndNrorg($this->notafiscal->getCdfilial(), $this->notafiscal->getNrlanctonf(), $this->notafiscal->getNrorg());

            if (empty($tipoDeConta)) {
                $validaConta[0] = true;
            } else {
                /** @var Itemproduto $produto */
                foreach ($produtosNota as $produto) {
                    foreach ($tipoDeConta as $tConta) {
                        if ($tConta['CDPRODUTO'] != $produto->getCdproduto()) {
                            $validaConta[0] = true;
                            break;
                        }
                    }

                }
            }

            $contapagar     = Contapagar::findOneByNrorgAndCdfilialAndNrlanctonf($this->notafiscal->getNrorg(), $this->notafiscal->getCdfilial(), $this->notafiscal->getNrlanctonf());
            $validaConta[1] = !$contapagar instanceof Contapagar;

            $autoforne      = Autoforne::getRepository()->getAutoForne($this->notafiscal->getCdfilial(), $this->notafiscal->getNrlanctonf(), $this->notafiscal->getNrorg());
            $validaConta[2] = empty($autoforne['CDTPCTPAGAR']);

            $tipoContaPagar = Tipoctpagar::getRepository()->verificaTipoContaPagar($this->notafiscal->getCdfilial(), $this->notafiscal->getNrlanctonf(), $this->notafiscal->getNrorg());
            $validaConta[3] = empty($tipoContaPagar['CDTPCTPAGAR']);

            if ($validaConta[0] && empty($cdtpctpagop) && !$validaConta[1] && $idcheck == 'U' && $validaConta[2] && $validaConta[3]) {
                AprovacaoException::tipoDeContaNaoParametrizado();
            }
        } elseif ($this->notafiscal->getIdentrsaidop() == Notafiscal::NOTA_FISCAL_SAIDA) {
            /** @var Contareceber $contarecer */
            $contarecer = Contareceber::findOneByCdfilialAndNrlanctonfAndNrorg($this->notafiscal->getCdfilial(), $this->notafiscal->getNrlanctonf(), $this->notafiscal->getNrorg());
            if($contarecer instanceof Contareceber){
                $contReceber = Contareceber::getRepository()->getContReceber($this->notafiscal->getCdfilial(), $this->notafiscal->getNrlanctonf(), $this->notafiscal->getNrorg());

                if(empty($cdtpctrecop) && empty($contReceber)){
                    AprovacaoException::tipoDeContaNaoParametrizado();
                }
            }
        }

        return true;
    }

    //
    public function validaConsisteFiscal(){
        $cdfilial = $this->notafiscal->getCdfilial();
        $nrlanctonf = $this->notafiscal->getNrlanctonf();
        $nrorg = $this->notafiscal->getNrorg();

        $consisteFiscal = Notafiscal::getRepository()->consisteFiscal($cdfilial, $nrlanctonf, $nrorg);

        if (is_array($consisteFiscal) && !empty($consisteFiscal)) {
            foreach ($consisteFiscal as $erro) {

                if (empty($erro['CDIMPOSTO'])) {

                    AprovacaoException::consistenciaFiscalSemImposto($erro['DSCONSISTENCIA'], $erro['CDARVPROD']);

                } else {

                    AprovacaoException::consistenciaFiscalComImposto($erro['DSCONSISTENCIA'], $erro['CDARVPROD'], $erro['CDIMPOSTO']);
                }
            }
        }
    }

    /**
     * Aprova as notas fiscais selecionadas
     *
     * @return bool|\Exception
     */
    public function aprovarNotaFiscal()
    {
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        $connection->beginTransaction();
        try {
            $erroDadosFinanceiros = '';

            $id = array(
                'nrorg'        => $this->notafiscal->getNrorg(),
                'cdtipooper'   => $this->notafiscal->getCdtipooper(),
                'identrsaidop' => $this->notafiscal->getIdentrsaidop());

            /** @var $tpoperac \Geral\Entities\Tipooperac */
            $tpoperac = \Geral\Entities\Tipooperac::getObject($id);

            if ($tpoperac->getIdutilvenc() != 'N') {
                $erroDadosFinanceiros = ProceduresNF::callProcedureGeraVencimento(
                    $this->notafiscal,
                    Notafiscal::STATUS_APROVADO
                );
            }

            if ($erroDadosFinanceiros['P_RESULTADO']) {
                switch ($erroDadosFinanceiros['P_RESULTADO']) {
                case 'NF_NOT_EXIST' :
                    AprovacaoException::notaFiscalNaoEncontrada();
                    break;
                case 'CNT_SUP_VRORC' :
                    AprovacaoException::contaSuperaValorOrcadoMes();
                    break;
                case 'INV_TP_OP' :
                    AprovacaoException::tipoOperacaoGeracaoInvalida();
                    break;
                case substr($erroDadosFinanceiros['P_RESULTADO'], 0, 16) == 'NOT_CENTCUSTFIL_' :
                    $str = str_split(substr($erroDadosFinanceiros['P_RESULTADO'], 16, 8), 4);
                    AprovacaoException::centroCustoNaoCadastradoFilial($str[0], $str[1]);
                    break;
                }
            }

            $nrlancctb = ProceduresNF::callProcedureContabilizaNotaFiscal($this->notafiscal);

            $erros = Util::verificaErrosContabilizacao($this->notafiscal->getCdempresa(), $nrlancctb['P_NRLANCCTB']);
            if ($erros) {
                AprovacaoException::existemErrosDeContabilizacao();
            }

            if ($this->notafiscal->getIdentrsaidop() == Notafiscal::NOTA_FISCAL_ENTRADA) {
                $ajustes = Ajctpagar::findByCdfilialAndNrlanctonfAndNrorg(
                    $this->notafiscal->getCdfilial(),
                    $this->notafiscal->getNrlanctonf(), $this->notafiscal->getNrorg()
                );
                /** @var $ajuste Ajctpagar */
                foreach ($ajustes as $ajuste) {
                    $nrlancctb = ProceduresNF::callProcedureAjuste(
                        $ajuste->getCdfilial(), $ajuste->getNrlanctonf(),
                        $ajuste->getNrseqvenpag(), $ajuste->getNrparcelapag()
                    );

                    $erros = Util::verificaErrosContabilizacao($this->notafiscal->getCdempresa(), $nrlancctb['P_NRLANCCTB']);
                    if ($erros) {
                        AprovacaoException::existemErrosDeContabilizacao();
                    }
                }
            }

            self::atualizaStatusNotaFiscalAprovada();
            $connection->commit();
            return true;
        } catch (\Exception $e) {
            $connection->rollback();
            //            TratamentoDeMensagens::exibeMensagensAcumuladas();
            return $e;
        }
    }

    /**
     * Atualiza o status da nota fiscal para APROVADA
     */
    private function atualizaStatusNotaFiscalAprovada()
    {
        $this->notafiscal->setIdstatus(Notafiscal::APROVADO);
        $this->notafiscal->setIdbloqueado(Notafiscal::NOTA_FISCAL_BLOQUEADA);
        $this->notafiscal->setCdoperapro(Ambiente::getCdOperador());
        $this->notafiscal->setDtaprovacao(DateUtil::getDataDeString(\Estoque\Util\Parametros::getDataProcessamento()));
        $this->notafiscal->setDtexportacao(null);
        $this->notafiscal->setIddocfinanc('N');
        $this->notafiscal->setIdexpnfep('N');
        $this->notafiscal->setIdexpnfemanaus('N');
        $this->notafiscal->setIdexpnfesanand('N');
        $this->notafiscal->setIdverifnfconf('N');

        if ($this->notafiscal->getTipoOperac()->getIdgeravencnfap() == 'S') {
            $this->notafiscal->setIdbloqnffin('B');
            $this->notafiscal->setDtliberacaofin(null);
        } elseif ($this->notafiscal->getTipoOperac()->getIdgeravencnfap() == 'N') {
            $this->notafiscal->setIdbloqnffin('L');
            $this->notafiscal->setDtliberacaofin(DateUtil::getDataAtual());
        }

        $this->notafiscal->setDtultatu(DateUtil::getDataAtual());
        $this->notafiscal->setCdoperultatu(Ambiente::getCdOperador());

        Registry::getEntityManager()->flush();
    }

    /**
     * Cancelar a aprovação das notas fiscais selecionadas
     *
     * @throws \Exception
     */
    public function cancelarNotaFiscalAprovada()
    {
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        $connection->beginTransaction();
        try {
            $erros = ProceduresNF::callProcedureCancelaDadosFinanceiro($this->notafiscal);
            if ($erros['P_RETORNO'] != 0) {
                throw new \Exception();
            }
            //                AprovacaoException::errosAoTentarCancelar();

            $this->deletaContabilizacao();

            self::atualizaStatusNotaFiscalCancelada();
            $connection->flush();
            $connection->commit();
            return $erros['P_RETORNO'];
        } catch (\Exception $e) {
            $connection->rollback();
            return $erros['P_RETORNO'];
            //            tk_abort($e->getMessage());
        }
    }

    /**
     * Deleta a contabilização da nota fiscal
     */
    private function deletaContabilizacao()
    {
        /** @var $lancamento Lanccont */
        $lancamentoContabil = Lanccont::findByCdempresaAndNrlanccont(
            $this->notafiscal->getCdempresa(),
            $this->notafiscal->getNrlanctbdoc()
        );
        if ($lancamentoContabil) {
            foreach ($lancamentoContabil as $lancamento)
                $lancamento->delete();
        }
    }

    /**
     * Atualiza os status da nota fiscal para CANCELADA
     */
    private function atualizaStatusNotaFiscalCancelada()
    {
        $this->notafiscal->setIdstatus(Notafiscal::PENDENTE);
        $this->notafiscal->setIdbloqueado(Notafiscal::NOTA_FISCAL_NAO_BLOQUEADA);
        $this->notafiscal->setCdopecancnf(Ambiente::getCdOperador());
        $this->notafiscal->setDtaprovacao(null);
        $this->notafiscal->setDtexportacao(null);
        $this->notafiscal->setIddocfinanc('N');
        $this->notafiscal->setIdexpnfep('N');
        $this->notafiscal->setIdexpnfemanaus('N');
        $this->notafiscal->setIdexpnfesanand('N');
        $this->notafiscal->setIdverifnfconf('N');
        $this->notafiscal->setNrseqcapadoc('');
        $this->notafiscal->setNrlanctbdoc(null);
        $this->notafiscal->setNrlanctbajus(null);

        $this->notafiscal->setDtultatu(DateUtil::getDataAtual());
        $this->notafiscal->setCdoperultatu(Ambiente::getCdOperador());

        Registry::getEntityManager()->flush();
    }
}